/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.models.transformers.bpmn2designer;

import javax.xml.namespace.QName;
import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.models.exceptions.TransformationException;
import org.ow2.orchestra.jaxb.bpmn.bpmndi.BPMNShape;
import org.ow2.orchestra.jaxb.bpmn.dc.Bounds;
import org.ow2.orchestra.jaxb.bpmn.di.DiagramElement;

/**
 * @author Loic Albertin
 */
public final class ShapeTransformer {

  /**
   * Retrieves the shape representing the graphical part of the 'semantic' BPMN element and extract positions from its.
   * The 'semantic' element may not have related shape.
   *
   * @param nodeId  The id of the BPMN 'semantic' element
   * @param model   The corresponding designer element
   * @param context The transformation context
   * @throws TransformationException Throws if the related {@link DiagramElement} is not a {@link BPMNShape} or if this {@link
   *                                 BPMNShape} doesn't contain {@link Bounds}
   */
  public static void bpmnToModel(final String nodeId, final AbstractElementWithPosition model, final Bpmn2DesignerContext context) throws
          TransformationException {
    final DiagramElement diagramElement = context.getDiagramElementsMap().get(
            new QName(context.getDefinitions().getTargetNamespace(), nodeId));
    if (diagramElement == null) {
      // No diagram element for this flow node this is not an error
      return;
    } else if (!(diagramElement instanceof BPMNShape)) {
      throw new TransformationException("BPMN conversion error: diagramElement linked to FlowNode " + nodeId
                                        + " is not a BPMNShape.");
    }
    final BPMNShape shape = (BPMNShape) diagramElement;
    if (shape.getBounds() == null) {
      throw new TransformationException("BPMN conversion error: malformed BPMN model, Shape " + shape.getId()
                                        + " should have Bounds.");
    }
    model.setX((int) shape.getBounds().getX());
    model.setY((int) shape.getBounds().getY());
  }
}
