/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.models.transformers.designer2bpmn;

import javax.xml.bind.JAXBElement;

import org.ow2.orchestra.designer.bpmn.model.GatewayModel;
import org.ow2.orchestra.designer.bpmn.model.GatewayType;
import org.ow2.orchestra.designer.models.exceptions.TransformationException;
import org.ow2.orchestra.jaxb.bpmn.TEventBasedGateway;
import org.ow2.orchestra.jaxb.bpmn.TEventBasedGatewayType;
import org.ow2.orchestra.jaxb.bpmn.TExclusiveGateway;
import org.ow2.orchestra.jaxb.bpmn.TGateway;
import org.ow2.orchestra.jaxb.bpmn.TParallelGateway;

/**
 * Transform {@link TGateway}s into {@link GatewayModel}s and vice versa
 *
 * @author Loic Albertin
 */
public final class GatewayTransformer {

  /**
   * Utility classes should not have public or default constructors.
   */
  private GatewayTransformer() {

  }

  /**
   * @param gatewayModel
   * @param context
   * @return
   */
  public static void modelToBpmn(final GatewayModel gatewayModel, final Designer2BpmnContext context) throws TransformationException {
    if (gatewayModel.getId() == null || gatewayModel.getId().equals("")) {
      throw new TransformationException("BPMN conversion error: A gateway has no id.");
    }
    TGateway tGateway;
    JAXBElement<? extends TGateway> jaxbElement;
    if (gatewayModel.getGatewayType() != null) {
      if (gatewayModel.getGatewayType().equals(GatewayType.PARALLEL)) {
        tGateway = new TParallelGateway();
        jaxbElement = context.getBpmnObjectFactory().createParallelGateway((TParallelGateway) tGateway);
      } else if (gatewayModel.getGatewayType().equals(GatewayType.EXCLUSIVE)) {
        tGateway = new TExclusiveGateway();
        jaxbElement = context.getBpmnObjectFactory().createExclusiveGateway((TExclusiveGateway) tGateway);
      } else if (gatewayModel.getGatewayType().equals(GatewayType.EXCLUSIVE_EVENT)) {
        TEventBasedGateway tEventGateway = new TEventBasedGateway();
        tEventGateway.setInstantiate(false);
        tEventGateway.setEventGatewayType(TEventBasedGatewayType.EXCLUSIVE);
        tGateway = tEventGateway;
        jaxbElement = context.getBpmnObjectFactory().createEventBasedGateway(tEventGateway);
      } else {
        throw new TransformationException(
            "BPMN conversion error: Unknown type: " + gatewayModel.getGatewayType() + " for gateway " + gatewayModel.getId()
            + ".");
      }
    } else {
      throw new TransformationException("BPMN conversion error: No type defined for gateway " + gatewayModel.getId() + ".");
    }
    tGateway.setId(gatewayModel.getId());
    tGateway.setName(gatewayModel.getName());

    context.getModelsMap().put(gatewayModel, tGateway);

    context.getTProcess().getFlowElements().add(jaxbElement);

    // Now deals with positions
    ShapeTransformer.modelToBpmn(gatewayModel, context);

  }

}
