/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.models.transformers.designer2bpmn;

import org.ow2.orchestra.designer.bpmn.model.PoolModel;
import org.ow2.orchestra.designer.models.exceptions.TransformationException;
import org.ow2.orchestra.jaxb.bpmn.Import;
import org.ow2.orchestra.jaxb.bpmn.Participant;
import org.ow2.orchestra.jaxb.bpmn.TCollaboration;

/**
 * Transform {@link org.ow2.orchestra.jaxb.bpmn.Participant}s into {@link org.ow2.orchestra.designer.bpmn.model.PoolModel}s and vice versa
 *
 * @author Guillaume Porcher
 */
public final class PoolTransformer {

  /**
   * Utility classes should not have public or default constructors.
   */
  private PoolTransformer() {

  }

  /**
   * @param poolModel
   * @param context
   * @return
   */
  public static void modelToBpmn(final PoolModel poolModel, final Designer2BpmnContext context) throws TransformationException {
    if (poolModel.getId() == null || poolModel.getId().equals("")) {
      throw new TransformationException("BPMN conversion error: A pool has no id.");
    }
    final Participant participant = new Participant();

    participant.setId(poolModel.getId());
    participant.setName(poolModel.getName());

    // Generate collaboration
    if (context.getCollaboration() == null) {
      final TCollaboration collaboration = new TCollaboration();
      context.setCollaboration(collaboration);
      context.getDefinitions().getRootElements().add(context.getBpmnObjectFactory().createCollaboration(collaboration));
    }
    context.getCollaboration().getParticipants().add(participant);

    // generate interfaces
    Designer2BpmnTransformer.generateInterfaces(context, participant, poolModel.getInterfaces().values());

    // Generate import
    if (poolModel.getWsdlPath() != null) {
      Import bpmnImport = new Import();
      bpmnImport.setImportType("http://schemas.xmlsoap.org/wsdl/");
      bpmnImport.setLocation(poolModel.getWsdlPath());
      bpmnImport.setNamespace(poolModel.getWsdlNamespace());

      context.getDefinitions().getImports().add(bpmnImport);
    }

    // Now deals with positions
    ShapeTransformer.modelToBpmn(poolModel, context);

  }


}
